<?php

namespace Drupal\couchbasedrupal;

use Drupal\Core\Site\Settings;

use Drupal\couchbasedrupal\CouchbaseBucket as Bucket;

/**
 * This is a service used to manage the connections
 * to the different couchbase servers.
 */
class CouchbaseManager {

  /**
   * Returns a 12 character length MD5.
   *
   * @param string $string
   * @return string
   */
  public static function shortMd5($string) {
    return substr(base_convert(md5($string), 16,32), 0, 12);
  }

  /**
   * Couchbase server settings.
   *
   * @var array
   */
  var $settings;

  /**
   * Settings
   *
   * @param Settings $settings
   */
  public function __construct(Settings $settings) {
    // Add a default for a local couchbase server.
    $this->settings = ['servers' =>
      ['default' => ['uri' => 'couchbase://127.0.0.1'] ]
    ];
    // Merge or override with custom settings.
    array_merge($this->settings, $settings::get('couchbasedrupal', array()));
  }

  /**
   * List of instanced clusters.
   *
   * @var \CouchbaseCluster[]
   */
  protected $clusters = array();

  /**
   * List of instanced buckets.
   *
   * @var Bucket[]
   */
  protected $buckets = array();

  /**
   * Retrieve a connection to the couchbase server.
   *
   * @param string $name
   * @return bool|\CouchbaseCluster
   */
  public function getCluster($name = 'default') {

    if (isset($this->clusters[$name])) {
      return $this->clusters[$name];
    }

    // For this service to work we need the CouchbaseCluster class.
    if (!class_exists(\CouchbaseCluster::class)) {
      return FALSE;
    }

    if (!isset($this->settings['servers'][$name]['uri'])) {
      return FALSE;
    }

    $uri = $this->settings['servers']['default']['uri'];

    $server = $this->ParseUri($uri);

    if (isset($server['username']) && isset($server['password'])) {
      $this->clusters[$name] = new \CouchbaseCluster($server['scheme'] . '://' . $server['host'], $server['username'], $server['password']);
    }
    else {
      $this->clusters[$name] = new \CouchbaseCluster($server['scheme'] . '://' . $server['host']);
    }

    return $this->clusters[$name];
  }

  /**
   * Retrieve a couchbase bucket.
   *
   * @param string $cluster
   * @param string $bucket
   * @return Bucket
   */
  public function getBucket($cluster = 'default', $bucket = 'default') {
    $cluster = $this->getCluster($cluster);
    if (!isset($this->buckets[$bucket])) {
      $this->buckets[$bucket] = new Bucket($cluster, $bucket);
    }
    return  $this->buckets[$bucket];
  }

  /**
   * Parse a couchbase connection URI.
   *
   * @param string $uri
   */
  protected function ParseUri($url) {
    $info = parse_url($url);
    if (!isset($info['scheme'], $info['host'])) {
      throw new \InvalidArgumentException('Minimum requirement: driver://host');
    }
    $info += array(
      'user' => '',
      'pass' => '',
      'fragment' => '',
    );

    $database = array(
      'scheme' => $info['scheme'],
      'username' => $info['user'],
      'password' => $info['pass'],
      'host' => $info['host'],
    );

    if (isset($info['port'])) {
      $database['port'] = $info['port'];
    }

    return $database;
  }
}
